@@ -2122,6 +2122,7 @@ bgp_init_pending_tx(struct bgp_channel *c)
   bpp->exporter = (struct rt_exporter) {
     .journal = {
       .loop = c->c.proto->loop,
+      .domain = dom.rtable,
       .item_size = sizeof(struct rt_export_item),
       .item_done = bgp_out_item_done,
     },
@@ -2210,6 +2211,10 @@ bgp_preexport(struct channel *C, rte *e)
   struct bgp_proto *src = bgp_rte_proto(e);
   struct bgp_channel *c = (struct bgp_channel *) C;
 
+  /* Reject everything on shutdown */
+  if (SHUTTING_DOWN)
+    return -1;
+
   /* Ignore non-BGP channels */
   if (C->class != &channel_bgp)
     return -1;
@@ -2421,6 +2426,10 @@ bgp_rt_notify(struct proto *P, struct channel *C, const net_addr *n, rte *new, c
   struct bgp_bucket *buck;
   struct rte_src *path;
 
+  /* Reject everything on shutdown */
+  if (SHUTTING_DOWN)
+    return;
+
   /* Ignore non-BGP channels */
   if (C->class != &channel_bgp)
     return;
